//-------------------http://www.opengroup.org/onlinepubs/009695399/functions/malloc.html
// RB Plugin Example
//-------------------

//includes

#include <ctype.h>
#include <stdarg.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>

#include "rb_plugin.h"

//-----------------------
// Functions
//-----------------------

int rbMalloc(int bytes) { return (int)malloc(bytes); }

int rbCalloc(int bytes) { return (int)calloc(1,bytes); }

void rbRealloc(int* address, int bytes) { *address = (int)realloc((void*)*address, bytes); }

void rbFree(int* address) { free((void*)*address); *address = 0; }

void rbMemCopy(int source, int dest, int bytes) { memcpy((void*)dest, (void*)source, bytes); }

void rbMemMove(int source, int dest, int bytes) { memmove((void*)dest, (void*)source, bytes); }

void rbMemClear(int address, int bytes) { memset((void*)address, 0, bytes); }

int rbMemCompare(int a, int b, int bytes) { return memcmp((void*)b, (void*)a, bytes); }

int rbGetInteger(int address) { return *((int*)address); }

void rbSetInteger(int address, int value) { *((int*)address) = value; }

float rbGetSingle(int address) { return *((float*)address); }

void rbSetSingle(int address, float value) { *((float*)address) = value; }

double rbGetDouble(int address) { return *((double*)address); }

void rbSetDouble(int address, double value) { *((double*)address) = value; }

Boolean rbGetBoolean(int address) { return *((Boolean*)address); }

void rbSetBoolean(int address, Boolean value) { *((Boolean*)address) = value; }

int rbGetShort(int address) { return *((short*)address); }

void rbSetShort(int address, int value) { *((short*)address) = value; }

int rbGetByte(int address) { return *((char*)address); }

void rbSetByte(int address, int value) { *((char*)address) = value; }

REALstring rbGetString(int address, int bytes) { return REALBuildString((char*)address, bytes); }

void rbSetString(int address, REALstring str) { const char* strptr = REALCString(str); strcpy((char*)address, strptr); }

int rbMemoryBlockToPointer(REALmemoryBlock mem) { return (int)REALMemoryBlockGetPtr(mem); }

REALmemoryBlock rbPointerToMemoryBlock(int address) { return REALPtrToMemoryBlock((void*)address); }

int rbStringToPointer(REALstring str) { return (int)REALCString(str); }

//-----------------------
// Add global RB methods
//-----------------------

static REALmethodDefinition pluginMethodArray[] = {

	{ (REALproc) rbMalloc, REALnoImplementation, "muMalloc(bytes as Integer) as Integer" },
	{ (REALproc) rbCalloc, REALnoImplementation, "muCalloc(bytes as Integer) as Integer" },
	{ (REALproc) rbRealloc, REALnoImplementation, "muRealloc(byref address as integer, bytes as Integer)" },
	{ (REALproc) rbFree, REALnoImplementation, "muFree(byref address as Integer)" },
	{ (REALproc) rbMemCopy, REALnoImplementation, "muMemCopy(source as integer, dest as Integer, bytes as integer)" },
	{ (REALproc) rbMemMove, REALnoImplementation, "muMemMove(source as integer, dest as Integer, bytes as integer)" },
	{ (REALproc) rbMemClear, REALnoImplementation, "muMemClear(address as integer, bytes as integer)" },
	{ (REALproc) rbMemCompare, REALnoImplementation, "muMemCompare(a as integer, b as Integer, bytes as integer) as Integer" },
	{ (REALproc) rbGetInteger, REALnoImplementation, "muGetInteger(address as Integer) as Integer" },
	{ (REALproc) rbSetInteger, REALnoImplementation, "muSetInteger(address as Integer, value as Integer)" },
	{ (REALproc) rbGetSingle, REALnoImplementation, "muGetSingle(address as Integer) as Single" },
	{ (REALproc) rbSetSingle, REALnoImplementation, "muSetSingle(address as Integer, value as Single)" },
	{ (REALproc) rbGetDouble, REALnoImplementation, "muGetDouble(address as Integer) as Double" },
	{ (REALproc) rbSetDouble, REALnoImplementation, "muSetDouble(address as Integer, value as Double)" },
	{ (REALproc) rbGetBoolean, REALnoImplementation, "muGetBoolean(address as Integer) as Boolean" },
	{ (REALproc) rbSetBoolean, REALnoImplementation, "muSetBoolean(address as Integer, value as Boolean)" },
	{ (REALproc) rbGetShort, REALnoImplementation, "muGetShort(address as Integer) as Integer" },
	{ (REALproc) rbSetShort, REALnoImplementation, "muSetShort(address as Integer, value as Integer)" },
	{ (REALproc) rbGetByte, REALnoImplementation, "muGetByte(address as Integer) as Integer" },
	{ (REALproc) rbSetByte, REALnoImplementation, "muSetByte(address as Integer, value as Integer)" },
	{ (REALproc) rbGetString, REALnoImplementation, "muGetString(address as Integer, bytes as Integer) as String" },
	{ (REALproc) rbSetString, REALnoImplementation, "muSetString(address as Integer, str as String)" },
	{ (REALproc) rbMemoryBlockToPointer, REALnoImplementation, "muMemoryBlockToPointer(mem as MemoryBlock) as Integer" },
	{ (REALproc) rbPointerToMemoryBlock, REALnoImplementation, "muPointerToMemoryBlock(address as integer) as MemoryBlock" },
	{ (REALproc) rbStringToPointer, REALnoImplementation, "muStringToPointer(str as String) as Integer" },
	
}; 

// -----------------------
//  Plugin Initialisation 
// -----------------------

void PluginEntry (void)
{
	
	// register global method definitions
	for (int i = 0; i < sizeof(pluginMethodArray) / sizeof(REALmethodDefinition); ++i) {
		REALRegisterMethod (&pluginMethodArray[i]);
	}
	
}
